from enum import StrEnum
from typing import Generator

import typer
from loguru import logger

from searchlauncher import open_in_browser, start_gui
from searchlauncher.settings import DEFAULT_SETTINGS_PATH, load_settings

cli = typer.Typer()

settings = load_settings()
Group = StrEnum("Group", list(settings.groups.keys()))  # type: ignore


def complete_groups(incomplete: str) -> Generator[tuple[str, str], None, None]:
    for group_name, group in settings.groups.items():
        if group_name.startswith(incomplete):
            yield group_name, ", ".join(group.sites)


@cli.command()
def search(
    query: str,
    group: Group = typer.Option(  # type: ignore
        None,
        "--group",
        "-g",
        autocompletion=complete_groups,
        case_sensitive=False,
    ),
) -> None:
    """
    Directly open search for the given query using all sites or the specified group.

    Searches all sites unless a group is specified with "-g" or "--group" (see "searchlauncher config").
    """
    open_in_browser(query, group_name=group)


@cli.command()
def config() -> None:
    """
    Open the config directory.

    You can modify the "toml" file to modify keyboard shortcuts, sites and groups.
    """
    settings_dir = str(DEFAULT_SETTINGS_PATH)
    logger.info(f"Opening settings directory: {settings_dir}")
    typer.launch(settings_dir, locate=True)


@cli.callback(invoke_without_command=True)
def run(
    ctx: typer.Context,
) -> None:
    """
    Run search launcher in background, waiting for shortcuts.

    See the "Commands" section below for more.
    """

    # This makes running just "searchlauncher" without any further commands run the main loop
    # https://github.com/tiangolo/typer/issues/18
    if ctx.invoked_subcommand is not None:
        # If there's any command, we don't run the main loop
        return
    start_gui()


if __name__ == "__main__":
    cli()



from functools import lru_cache
from pathlib import Path
from typing import Any

from loguru import logger
from pydantic import BaseModel, BaseSettings, HttpUrl
from pydantic.env_settings import SettingsSourceCallable
from tomlkit import load

# this file → this folder → settings folder → default.toml
DEFAULT_SETTINGS_PATH = Path(__file__).parent / "settings" / "default.toml"


class SearchGroup(BaseModel):
    shortcut: str = ""
    sites: list[str] = []
    groups: list[str] = []


def toml_settings(settings: BaseSettings) -> dict[str, Any]:
    """
    A simple settings source that loads variables from a JSON file
    at the project's root.

    Here we happen to choose to use the `env_file_encoding` from Config
    when reading `config.json`
    """
    logger.debug(f"Loading settings from {DEFAULT_SETTINGS_PATH}")

    with open(DEFAULT_SETTINGS_PATH) as f:
        settings = load(f)
    return settings


class Settings(BaseSettings):
    shortcut: str
    sites: dict[str, HttpUrl]
    groups: dict[str, SearchGroup]

    class Config:
        @classmethod
        def customise_sources(
            cls,
            init_settings: SettingsSourceCallable,
            env_settings: SettingsSourceCallable,
            file_secret_settings: SettingsSourceCallable,
        ) -> tuple[SettingsSourceCallable, ...]:
            return init_settings, toml_settings


@lru_cache()
def load_settings(**kwargs) -> Settings:  # type: ignore
    return Settings(**kwargs)


if __name__ == "__main__":
    settings = load_settings()
    print(settings)



import platform
from tkinter import Entry, StringVar, Tk

import keyboard
from loguru import logger

from searchlauncher.settings import load_settings

from .search import open_in_browser

SHORTCUT = "ctrl+shift+f"
QUERY_WIDTH = 25
FONT = "Arial 30"

# Our popup window will get focused after this many milliseconds
# FOCUS_DELAY = 42
FOCUS_DELAY = 69
# FOCUS_DELAY = 420


def start_search(query: StringVar, group_name: str | None, root: Tk) -> None:
    query_str: str = query.get()

    # To handle invalid queries:
    # if not(query.startswith('http://') or query.startswith('https://')):
    #     messagebox.showerror("Invalid query", "Must start with http:// or https://")
    #     return

    open_in_browser(query_str, group_name)

    root.destroy()


def show_launcher(group_name: str) -> None:
    root = Tk()
    root.title(f"Search launcher – {group_name}")

    # center on screen
    root.eval("tk::PlaceWindow . center")

    # create the user input for query
    query = StringVar()
    entry = Entry(root, width=QUERY_WIDTH, textvariable=query, font=FONT)
    entry.pack()

    # submit on enter, quit on escape
    root.bind("<Return>", lambda _: start_search(query, group_name, root))
    root.bind("<Escape>", lambda _: root.destroy())

    # move into foreground
    root.attributes("-topmost", True)
    # obtain focus – if the FOCUS_DELAY is too short, root won't get focus (foreground) at all
    root.after(FOCUS_DELAY, lambda: entry.focus_force())

    root.mainloop()


def start_gui() -> None:
    if platform.system() == "Windows":
        logger.debug("Fixing resolution scaling on Windows")
        from ctypes import windll  # type: ignore

        # Fix blurry font with resolution scaling on Windows:
        windll.shcore.SetProcessDpiAwareness(1)

    settings = load_settings()

    for group_name, group in settings.groups.items():

        # a group does not need to have a shortcut, it could just be used by a different group
        if not group.shortcut:
            continue

        logger.info(f"Search {group_name} with '{group.shortcut}'")
        keyboard.add_hotkey(group.shortcut, show_launcher, args=(group_name,))

    # wait() loops automatically:
    keyboard.wait()


if __name__ == "__main__":
    start_gui()
